
# q-expansion of newform 1134.2.a.h, downloaded from the LMFDB on 16 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1134
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [0], [3], [1], [6], [2], [-6], [-7], [-3], [-6], [2], [2], [0], [2], [0], [-6], [0], [5], [8], [-3], [2], [5], [-12], [0], [2], [-9], [-10], [12], [-10], [-15], [17], [9], [-6], [5], [6], [23], [-13], [2], [0], [6], [-18], [-25], [9], [17], [-18], [14], [8], [-28], [15], [-1], [-9], [15], [8], [-3], [-18], [21], [9], [-28], [-16], [27], [-19], [3], [-25], [-12], [-10], [-18], [26], [-22], [24], [26], [18], [3], [8], [14], [2], [18], [-24], [26], [-3], [32], [-15], [-10], [-12], [14], [8], [-18], [-33], [29], [33], [-13], [-12], [6], [29], [-18], [32], [-12], [-30], [-24], [-13], [38], [32], [24], [33], [18], [32], [-4], [3], [0], [24], [14], [-22], [8], [-42], [-7], [-7], [-27], [-4], [0], [36], [-42], [5], [-37], [-42], [6], [47], [18], [-52], [36], [8], [29], [26], [-36], [-31], [26], [42], [14], [51], [20], [-3], [30], [-16], [24], [8], [-36], [-34], [0], [35], [-54], [-4], [-9], [-22], [0], [-34], [-24], [32], [-15], [11], [0], [38], [-21], [24], [-42], [17], [15], [-6], [-18], [-40], [-55], [-22], [-15], [30], [23], [3], [-19], [56], [60], [-58], [60], [-22], [29], [35], [0], [14], [-54], [9], [30], [-46], [-37], [11], [60], [-52], [18], [20], [21], [-30], [60], [41], [56], [-6], [0], [-18], [14], [53], [-4], [-63], [-45], [14], [36], [-30], [-31], [11], [-30], [-4], [33], [-60], [2], [-67], [-12], [24], [36], [20], [14], [15], [-16], [-48], [56], [-24], [51], [8], [12], [-40], [-7], [-19], [6], [-73], [54], [68], [42], [0], [42], [-24], [20], [-4], [26], [-18], [24], [26], [33], [-40], [33], [68], [0], [27], [-16], [-6], [-36], [50], [-49], [51], [-52], [-64], [60], [-10], [29], [12], [-4], [-6], [48], [-4], [24], [14], [20], [32], [80], [-22], [8], [-84], [14], [14], [75], [0], [-16], [6], [65], [11], [-69], [-19], [-12], [74], [6], [-36], [-39], [-21], [-12], [-10], [-3], [-22], [33], [36], [68], [-64], [-24], [-61], [-36], [-58], [-1], [-48], [5], [12], [-34], [-18], [66], [-78], [-46], [-21], [-40], [9], [45], [-70], [6], [44], [-13], [-6], [23], [3], [20], [-16], [8], [-21], [-54], [2], [60], [-28], [-9], [2], [-33], [-82], [18], [26], [20], [14], [-15], [78], [-19], [36], [-30], [83], [8], [30], [90], [23], [5], [-72], [-28], [-28], [-6], [90], [-69], [-45], [0], [56], [-90], [-12], [-60], [-64], [-19], [-42], [8], [41], [60], [56], [87], [0], [14], [-82], [-63], [-33], [32], [-30], [8], [69], [-9], [-97], [-39], [53], [72], [8], [-31], [2], [0], [2], [3], [-42], [-1], [-60], [-25], [-88], [66], [-70], [66], [5], [-57], [32], [24], [57], [-58], [-70], [3], [89], [36], [-28], [18], [-12], [14], [-31], [-63], [-24], [-10], [42], [-24], [90], [77], [-33], [-39], [-16], [66], [-24], [75], [11], [-66]]
